/*!
 * @vueblocks/vue-use-core v0.2.2
 * (c) 2021 xiaoluoboding
 * @license MIT
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi'), require('lodash.debounce'), require('@juggle/resize-observer'), require('screenfull'), require('lodash.throttle'), require('keymaster')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi', 'lodash.debounce', '@juggle/resize-observer', 'screenfull', 'lodash.throttle', 'keymaster'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VueUseUtilities = {}, global.VueDemi, global.lodash.debounce, global.ResizeObserver, global.screenfull, global.lodash.throttle, global.keymaster));
}(this, (function (exports, vueDemi, debounce, resizeObserver, screenfull, throttle, keymaster) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);
  var screenfull__default = /*#__PURE__*/_interopDefaultLegacy(screenfull);
  var throttle__default = /*#__PURE__*/_interopDefaultLegacy(throttle);
  var keymaster__default = /*#__PURE__*/_interopDefaultLegacy(keymaster);

  const useDebouncedRef = (value, delay = 200) => {
      return vueDemi.customRef((track, trigger) => {
          return {
              get() {
                  track();
                  return value;
              },
              set: debounce__default['default']((newValue) => {
                  value = newValue;
                  trigger();
              }, delay)
          };
      });
  };

  /**
   * It immediately detects when an element resizes and provides accurate sizing information back to the handler.
   * @param target
   * @param callback
   * @param options
   */
  const useResizeObserver = (target, callback, options) => {
      let ro;
      const targetRef = vueDemi.ref(target);
      const isSupported = window && 'ResizeObserver' in window;
      const stopWatch = vueDemi.watch(targetRef, (newVal) => {
          if (isSupported && window && newVal) {
              ro = new resizeObserver.ResizeObserver(callback);
              ro.observe(newVal, options);
          }
      }, { immediate: true });
      const stop = () => {
          ro && ro.disconnect();
          ro = undefined;
          stopWatch();
      };
      return {
          stop
      };
  };

  const isBoolean = (val) => typeof val === 'boolean';
  const isClient = ![typeof window, typeof document].includes('undefined');
  const isElement = (val) => typeof val === 'object' && !!val.tagName;

  const useElementSize = (target, options) => {
      const width = vueDemi.ref(isElement(target) ? target.clientWidth : 0);
      const height = vueDemi.ref(isElement(target) ? target.clientHeight : 0);
      useResizeObserver(target, ([entry]) => {
          width.value = entry.contentRect.width;
          height.value = entry.contentRect.height;
      }, options);
      return { width, height };
  };

  /**
   * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
   * @param fn
   * @param sync if set to false, it will run in the nextTick() of Vue
   */
  function tryOnMounted(fn, sync = true) {
      if (vueDemi.getCurrentInstance()) {
          vueDemi.onMounted(fn);
      }
      else if (sync) {
          fn();
      }
      else {
          vueDemi.nextTick(fn);
      }
  }
  /**
   * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
   * @param fn
   */
  function tryOnUnmounted(fn) {
      if (vueDemi.getCurrentInstance())
          vueDemi.onUnmounted(fn);
  }

  /**
   * Reactive Fullscreen API build on top of screenfull.js
   * @param {HTMLElement} target
   * @return {boolean} isFullscreen
   * @return {function} toggleFullscreen
   */
  const useFullscreen = (target) => {
      const targetRef = vueDemi.ref(target || document.querySelector('html'));
      const isFullscreen = vueDemi.ref(false);
      const toggleFullscreen = () => {
          screenfull__default['default'].isEnabled && screenfull__default['default'].toggle(targetRef.value);
      };
      tryOnMounted(() => {
          if (screenfull__default['default'].isEnabled) {
              screenfull__default['default'].on('change', () => {
                  isFullscreen.value = screenfull__default['default'].isFullscreen;
              });
          }
      });
      return {
          isFullscreen,
          toggleFullscreen
      };
  };

  function useReactiveState(getter) {
      const state = vueDemi.reactive(getter());
      const update = () => {
          Object.assign(state, getter());
      };
      return [state, update];
  }

  /**
   * Reactive Resize API, When window resize, do someting.
   * @param {function} fn
   * @param {object} options
   * @return {function} dispatchResize
   */
  const useResizeEvent = (fn, options = { useCapture: false, useThrottle: null, delay: 500 }) => {
      const { useCapture = false, useThrottle = null, delay = 500 } = options;
      const dispatchResize = () => {
          const e = document.createEvent('Event');
          e.initEvent('resize', true, true);
          window.dispatchEvent(e);
      };
      const waitFn = isBoolean(useThrottle)
          ? useThrottle
              ? throttle__default['default'](fn, delay)
              : debounce__default['default'](fn, delay)
          : fn;
      tryOnMounted(() => {
          window.addEventListener('resize', waitFn, useCapture);
      });
      tryOnUnmounted(() => {
          window.removeEventListener('resize', waitFn, useCapture);
      });
      return {
          dispatchResize
      };
  };

  const bindKeyHandler = (fn) => {
      return (...args) => {
          fn(...args);
          return false;
      };
  };
  /**
   * Given shortcut a new bind function
   */
  const shortcut = {
      bind: (seed, func) => keymaster__default['default'](seed, bindKeyHandler(func)),
      ...keymaster__default['default']
  };
  /**
   * Bind shortcut onMounted & unbind shortcut automatically onUnmounted in setup hooks
   * or
   * Unbind shortcut before bind shortcut without setup hooks
   * @param {Object} keymap
   * @param {Object} vm
   * @return {function} toggle
   */
  const useShortcut = (keymap, vm) => {
      let sync = true;
      if (!vm) {
          vm = vueDemi.getCurrentInstance();
      }
      else {
          sync = false;
          unbindKeys();
      }
      function bindKeys() {
          Object.keys(keymap).forEach(key => {
              shortcut.bind(key, keymap[key]);
          });
      }
      function unbindKeys() {
          Object.keys(keymap).forEach(key => {
              shortcut.unbind(key);
          });
      }
      tryOnMounted(() => bindKeys(), sync);
      tryOnUnmounted(unbindKeys);
      return shortcut;
  };

  /**
   * Reactive window size
   * @param {object} options
   * @return {Ref<number>} WindowSize
   */
  const useWindowSize = (options) => {
      const width = vueDemi.ref(isClient ? window.innerWidth : 0);
      const height = vueDemi.ref(isClient ? window.innerHeight : 0);
      const onResize = () => {
          width.value = window.innerWidth;
          height.value = window.innerHeight;
      };
      const { dispatchResize } = useResizeEvent(onResize, options);
      vueDemi.onMounted(dispatchResize);
      return { width, height };
  };

  exports.tryOnMounted = tryOnMounted;
  exports.tryOnUnmounted = tryOnUnmounted;
  exports.useDebouncedRef = useDebouncedRef;
  exports.useElementSize = useElementSize;
  exports.useFullscreen = useFullscreen;
  exports.useReactiveState = useReactiveState;
  exports.useResizeEvent = useResizeEvent;
  exports.useResizeObserver = useResizeObserver;
  exports.useShortcut = useShortcut;
  exports.useWindowSize = useWindowSize;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
